//************************************************************************************
// sig2knockd.cpp : Defines the entry point for the console application.
//
// Version 0.2
// Copyright (c) 2004, SIG^2 (www.security.org.sg)
// All rights reserved.
//
// Linux Coding by Chew Keong TAN
//
// LICENSE TERMS
//
// The free distribution and use of this software in both source and binary 
// form is allowed (with or without changes) provided that:
//
//   1. distributions of this source code include the above copyright 
//      notice, this list of conditions and the following disclaimer;
//
//   2. distributions in binary form include the above copyright
//      notice, this list of conditions and the following disclaimer
//      in the documentation and/or other associated materials;
//
//   3. the copyright holder's name is not used to endorse products 
//      built using this software without specific written permission. 
//
// DISCLAIMER
//
// This software is provided 'as is' with no explicit or implied warranties
// in respect of its properties, including, but not limited to, correctness 
// and fitness for purpose.
//
//************************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <limits.h>
#include <netdb.h>
#include <arpa/inet.h>

#include <net/if.h>
#include <termios.h>
#include <time.h>
#include "md5.h"
#include "randomGen.h"
#include "windowsdef.h"
#include "clientProcessPkt.h"
#include "version.h"
#include "user.h"
#include "aes.h"
#include "aesCBC.h"


DWORD gOurIPAddress;


//************************************************************************************
// Gets User ID and Password from Console
//
//************************************************************************************

void getUserIDPassword(char *userID, int userIDSize, char *password, int passwordSize)
{
	struct termios origFlags, newFlags;
	char *userNamePrompt = "User Name: ";
	char *userPasswordPrompt = "Password: ";


	memset(userID, 0, userIDSize);
	write(1, userNamePrompt, strlen(userNamePrompt));
	read(0, userID, userIDSize - 1);

	userID[userIDSize - 1] = 0;
	int i;
	for(i = userIDSize - 1; i >= 0; i--)
	{
		if(userID[i] == 0x0a || userID[i] == 0x0d)
			userID[i] = 0;
	}

	// turn off echo
	tcgetattr(0, &origFlags);

	newFlags = origFlags;
	newFlags.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL | ECHOCTL | ECHOPRT | ECHOKE);

	tcsetattr(0, TCSANOW, &newFlags);

	// clear the buffer
	fd_set fset;
	FD_ZERO(&fset);
	FD_SET(0, &fset);
	struct timeval tv;
	tv.tv_sec = 1;
	tv.tv_usec = 0;
	int status = select(1, &fset, NULL, NULL, &tv);
	
	if(status)
	{
		char buffer[1024];
	        read(0, buffer, sizeof(buffer));
	}
	                                                                                                                 
	
	memset(password, 0, sizeof(password));	
	write(1, userPasswordPrompt, strlen(userPasswordPrompt));	
	read(0, password, passwordSize - 1);


	password[passwordSize - 1] = 0;
	for(i = passwordSize - 1; i >= 0; i--)
	{
		if(password[i] == 0x0a || password[i] == 0x0d)
			password[i] = 0;
	}
					
	// turn on echo
	tcsetattr(0, TCSANOW, &origFlags);
}


//************************************************************************************
// Generates user's password hash
// MD5(userid + password)
//
// outPasswordHash must have SIZEOF_USERPASSWORD_HASH of storage
//
//************************************************************************************

void compUserPasswordHash(char *userID, char *userPassword, unsigned char *outPasswordHash)
{
	MD5Context ctx;
	MD5Init(&ctx);
	MD5Update(&ctx, (unsigned char *)userID, strlen(userID));
	MD5Update(&ctx, (unsigned char *)userPassword, strlen(userPassword));
						
	MD5Final(outPasswordHash, &ctx);

}


void printUsage(char *exeName)
{
	printf("\nUsage: %s <server IP> <server Port>\n\n", exeName);
	return;
}


int main(int argc, char* argv[])
{
	printf("SIG^2 KnockC Version %s Copyright (c) 2004 SIG^2 (www.security.org.sg)\n"
	       "Linux Coding by Chew Keong TAN\n\n", PROG_VERSION);

	if(argc != 3)
	{
		printUsage(argv[0]);
		return 1;
	}

	if(!createRawSendSocket())
		return 1;

	// drop priv
	setreuid(getuid(), getuid());
	setregid(getgid(), getgid());
	
	char *destIP = argv[1];
	WORD destPort = atoi(argv[2]);

	if(!initRand())
	{
		printf("Error initializing random generator!\n");
		return 1;
	}
	
	// get our IP address
	getOutInterfaceIP(inet_addr(destIP), &gOurIPAddress);

	char userID[SIZEOF_USERID];
	char userPassword[SIZEOF_USERPASSWORD];
	unsigned char userPasswordHash[SIZEOF_USERPASSWORD_HASH];

	getUserIDPassword(userID, sizeof(userID), userPassword, sizeof(userPassword));
	if(strlen(userID) == 0 || strlen(userPassword) == 0)
	{
		printf("\nInvalid UserID or Password.\n");
		return 1;
	}
	printf("\n\n");

	compUserPasswordHash(userID, userPassword, userPasswordHash);
	memset(userPassword, 0, sizeof(userPassword));		// clear out

	PORTSEQ_REQ pq, netorder_pq;	
	generateKnockSequence(&pq, &netorder_pq, userID, userPasswordHash);	

	// send encrypt knock seq
	if(doUDPPortKnockReq(destIP, destPort, &netorder_pq, userID, userPasswordHash))
	{
		if(doSYNPortKnock(destIP, &pq))		// send knocks
		{			
			PORTREQ_QUERY prq, netorder_prq;

			generateKnockResultQuery(&prq, &netorder_prq, userID, userPasswordHash, pq.timeStamp);

			WORD allocPort;
			WORD portOpenTime;
			WORD reqType;

			// send query and wait for reply
			if(doUDPPortKnockQuery(destIP, destPort, &prq, &netorder_prq, userID, userPasswordHash, 
				                   &allocPort, &portOpenTime, &reqType))
			{
				memset(userID, 0, sizeof(userID));
				memset(userPasswordHash, 0, sizeof(userPasswordHash));

				printf("\nDoor is open at %s Port %d for %d seconds.\n", 
					   destIP, allocPort, portOpenTime);

				for(int timeCount = portOpenTime; timeCount > 0; timeCount--)
				{
					printf("%d, ", timeCount);
					fflush(stdout);
					sleep(1);
				}
				printf("\n");
			}
			else
			{
				printf("\nSorry, the door is still closed.\n");
			}
		}
	}
		
	return 0;
}
