//************************************************************************************
// Copyright (c) 2006, Olivier ROCH VILATO, Chew Keong TAN
// All rights reserved.
//
// Win32/Linux Coding by Chew Keong TAN and Olivier ROCH VILATO
//
// This file is part of DrKnock (based on sig2knock).

// DrKnock is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// DrKnock is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with DrKnock; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//************************************************************************************

#include <stdio.h>
#if !defined(WIN32)
  #include <string.h>
  #include <stdlib.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include "windowsdef.h"
#else
  #include <windows.h>
#endif

#include "userRecord.h"

/*****************************************************************************************
# Configuration file for WinKnockD

UDP_PORT				=	1000
FORWARD_TO_IP				=	127.0.0.1
FORWARD_TO_PORT				=	22
SINGLECONN_PORTOPEN_TIME		=	20			in seconds
PKTFILTER_INTERFACE			=	eth0

*****************************************************************************************/

typedef struct _SERVER_CONFIG
{
	char *configName;
	char configValue[200];
} SERVER_CONFIG;


SERVER_CONFIG serverConf[] = {
	{ "UDP_PORT",				"1000" },
	{ "FORWARD_TO_IP",			"127.0.0.1" },
	{ "FORWARD_TO_PORT",			"22" },
	{ "SINGLECONN_PORTOPEN_TIME",		"20" },
	{ "PKTFILTER_INTERFACE",		"" } };		


void printConfig(void)
{
	for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
	{
		printf("%-25s = %s\n", serverConf[i].configName, serverConf[i].configValue);
	}
}

WORD getConfigUDPPort(void)
{
	for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
	{
		if(strcmp(serverConf[i].configName, "UDP_PORT") == 0)
			return atoi(serverConf[i].configValue);
	}
	return 0;
}


DWORD getConfigForwardToIP(void)
{
	for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
	{
		if(strcmp(serverConf[i].configName, "FORWARD_TO_IP") == 0)
			return inet_addr(serverConf[i].configValue);
	}
	return 0;
}

WORD getConfigForwardToPort(void)
{
	for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
	{
		if(strcmp(serverConf[i].configName, "FORWARD_TO_PORT") == 0)
			return atoi(serverConf[i].configValue);
	}
	return 0;
}

WORD getConfigSingleConnPortOpenTime(void)
{
	for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
	{
		if(strcmp(serverConf[i].configName, "SINGLECONN_PORTOPEN_TIME") == 0)
			return atoi(serverConf[i].configValue);
	}
	return 0;
}

char *getPktFilterInterface(void)
{
	for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
	{
		if(strcmp(serverConf[i].configName, "PKTFILTER_INTERFACE") == 0)
			return serverConf[i].configValue;
	}
	return NULL;
}


BOOL loadServerConfig(char *filename)
{
	FILE *fp;

	fp = fopen(filename, "r");
	if(fp)
	{
		char buffer[257];
		while(fgets(buffer, sizeof(buffer), fp))
		{
			strTrim(buffer);
			if(strlen(buffer) && buffer[0] != '#')		// comment char = #
			{
				if(countTokenChar(buffer, '=') == 1)
				{					
					char *ptr = buffer;
					while(*ptr != '=')
						ptr++;

					*ptr = 0;
					char *ptr2 = ptr + 1;
					
					char *ptr3 = ptr2;
					while(*ptr3)
					{
						if(*ptr3 == '#')		// handle comments on each line
						{
							*ptr3 = 0;
							break;
						}
						ptr3++;
					}

					strTrim(buffer);
					strTrim(ptr2);

					for(int i = 0; i  < sizeof(serverConf) / sizeof(SERVER_CONFIG); i++)
					{
						if(strcmp(buffer, serverConf[i].configName) == 0)
						{
							if(strlen(ptr2))
							{
								memset(serverConf[i].configValue, 0, sizeof(serverConf[i].configValue));
								strncpy(serverConf[i].configValue, ptr2, sizeof(serverConf[i].configValue) - 1);
							}
						}
					}
				}
			}
		}
		fclose(fp);
		return TRUE;
	}

	return FALSE;
}
